@@ -119,6 +119,21 @@ int suite_vcn_tests_clean();
 + */
 extern CU_TestInfo vcn_tests[];
 
+/**
+ * Initialize uvd enc test suite
+ */
+int suite_uvd_enc_tests_init();
+
+/**
+ * Deinitialize uvd enc test suite
+ */
+int suite_uvd_enc_tests_clean();
+
+/**
+ * Tests in uvd enc test suite
+ */
+extern CU_TestInfo uvd_enc_tests[];
+
 /**
  * Helper functions
  */
